using System.Security.Cryptography.X509Certificates;
using Autofac;
using FitConnect.Services;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;

namespace FitConnect;

public static class DiContainer {
    public static void DisposeContainer() {
        _container?.Dispose();
        _container = null;
    }

    private static IContainer? _container;

    public static Autofac.IContainer GetContainer(FitConnectEndpoints endpoints,
        X509Certificate2? senderCertificate,
        X509Certificate2? receiverCertificate,
        ILogger? logger = null) =>
        _container ??= BuildContainer(endpoints, senderCertificate, receiverCertificate, logger);

    private static IContainer
        BuildContainer(FitConnectEndpoints endpoints,
            X509Certificate2? senderCertificate,
            X509Certificate2? receiverCertificate,
            ILogger? logger = null) {
        var builder = new ContainerBuilder();

        if (logger == null)
            logger = NullLoggerFactory.Instance.CreateLogger("FitConnect");

        builder.RegisterInstance(endpoints).As<FitConnectEndpoints>();
        builder.RegisterInstance(logger).As<ILogger>();

        builder.RegisterType<Client>();
        builder.Register(c => new OAuthService(endpoints.TokenUrl)).As<IOAuthService>();
        builder.Register(c => new SubmissionService(endpoints.SubmissionUrl))
            .As<ISubmissionService>();
        builder.Register(c => new RouteService(endpoints.RoutingUrl)).As<IRouteService>();

        builder.Register(c => new Sender(
            endpoints,
            senderCertificate!,
            new OAuthService(endpoints.TokenUrl),
            new SubmissionService(endpoints.SubmissionUrl),
            new RouteService(endpoints.RoutingUrl),
            logger)).As<Sender>();

        builder.Register(c => new Subscriber(
            endpoints,
            senderCertificate!,
            new OAuthService(endpoints.TokenUrl),
            new SubmissionService(endpoints.SubmissionUrl),
            new RouteService(endpoints.RoutingUrl),
            logger)).As<Subscriber>();


        return builder.Build();
    }
}
