using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
/// The fluent implementation for the <see cref="Sender"/> to encapsulate the FitConnect API.
/// </summary>
/// <example>
/// Reference for FluentSender
/// <code>
/// client.Sender
///     .Authenticate(clientId!, clientSecret!)
///     .CreateSubmission(new Submission { Attachments = new List() })
///     .UploadAttachments()
///     .SendSubmission(new Metadata(), new Data());
/// </code>
/// </example>
public class FluentSender : Sender, IFluentApi<FluentSender> {
    private OAuthAccessToken? token;


    private enum State {
        Authenticate,
        CreateSubmission,
        UploadAttachments,
        SendSubmission,
        Complete
    }

    private State _nextStep = State.Authenticate;

    public FluentSender(FitConnectEndpoints endpoints,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        X509Certificate2 certificate,
        ILogger? logger = null) : base(endpoints, certificate, oAuthService, submissionService,
        routeService, logger) {
        PublicKey = certificate.PublicKey;
    }

    public PublicKey PublicKey { get; set; }
    public CreateSubmissionDto? NewSubmission { get; set; }
    public Submission Submission { get; set; }


    /// <summary>
    ///     Authenticates with the FitConnect API.
    /// </summary>
    /// <param name="clientId">The client id from the submission service</param>
    /// <param name="clientSecret">The client secret</param>
    /// <param name="scope">Optional scope for the credentials</param>
    /// <returns></returns>
    public FluentSender Authenticate(string clientId, string clientSecret, string? scope = null) {
        if (_nextStep != State.Authenticate)
            throw new InvalidOperationException("Client not ready for authentication.");

        Owner.ClientId = clientId;
        Owner.ClientSecret = clientSecret;

        token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;
        _nextStep = State.CreateSubmission;
        return this;
    }

    /// <summary>
    ///     Creates a new <see cref="Submission" /> on the FitConnect server.
    /// </summary>
    /// <param name="submission" cref="Submission">The <see cref="Submission"/> body to be sent</param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException">If sender is not authenticated</exception>
    /// <exception cref="ArgumentException">If submission is not ready to be sent</exception>
    public FluentSender CreateSubmission(Submission submission) {
        if (_nextStep != State.CreateSubmission)
            throw new AuthenticationException(
                "Can only create a submission after authentication");
        Submission = submission;

        if (!Submission.Destination.Valid)
            throw new ArgumentException("Not all destination values are set");

        var leikaKey = Submission.Destination.LeikaKey;
        var ags = Submission.Destination.Ags;
        var ars = Submission.Destination.Ars;
        var areaId = Submission.Destination.AreaId;
        submission.DestinationId =
            RouteService.GetDestinationIdAsync(leikaKey, ags, ars, areaId).Result;

        if (!submission.IsSubmissionReadyToAdd(out var errorMessage))
            throw new ArgumentException($"Submission is not ready to add. {errorMessage}");

        NewSubmission = AddSubmission(submission);
        _nextStep = State.UploadAttachments;

        return this;
    }


    /// <summary>
    /// Posts the <see cref="Attachment"/>s to the FitConnect API. The attachment ids are sent with creating the submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSender UploadAttachments() {
        if (_nextStep != State.UploadAttachments)
            throw new InvalidOperationException(
                "Can only upload attachments after a submission has been created");
        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        _nextStep = State.SendSubmission;
        if (Submission.Attachments.Count == 0)
            return this;

        var encryptedAttachments = Encrypt(PublicKey, Submission.Attachments);
        UploadAttachmentsAsync(encryptedAttachments).Wait();
        return this;
    }

    /// <summary>
    /// Sending the submission and finishing the process.
    /// <para>
    /// Finishing the submission by transmitting <see cref="Metadata"/> and <see cref="Data"/>.
    /// </para> 
    /// </summary>
    /// <param name="metadata"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    /// <remarks>This call finishes the submission process</remarks>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSender SendSubmission(Metadata? metadata, Data? data = null) {
        if (_nextStep != State.SendSubmission)
            throw new InvalidOperationException(
                "Can only send a submission after attachments have been uploaded");

        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        Submission.Metadata = metadata;
        Submission.Data = data;
        var submitSubmissionDto = CreateSubmitSubmissionDto(Submission);
        SubmissionService.SubmitSubmission(Submission.Id, submitSubmissionDto);

        _nextStep = State.Complete;

        return this;
    }
}
