using System.Buffers.Text;
using System.Security.Authentication;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace FitConnect;

/// <summary>
/// Fluent API for the FitConnect.Subscriber
/// </summary>
public class FluentSubscriber : Subscriber, IFluentApi<FluentSubscriber> {
    private OAuthAccessToken? _token;

    #region Result values

    public Dictionary<string, IEnumerable<string>>? Submissions { get; private set; }
    public long? AvailableSubmissions { get; private set; }
    public Submission? Submission { get; set; }

    #endregion

    public FluentSubscriber(FitConnectEndpoints endpoints,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        X509Certificate2 certificate,
        ILogger? logger = null) : base(endpoints, certificate, oAuthService, submissionService,
        routeService, logger) {
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    public FluentSubscriber
        Authenticate(string clientId, string clientSecret, string? scope = null) {
        Owner.ClientId = clientId;
        Owner.ClientSecret = clientSecret;

        _token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;

        return this;
    }

    /// <summary>
    /// Receives a specific submission from the server.
    /// </summary>
    /// <param name="submissionId"></param>
    /// <returns></returns>
    public FluentSubscriber GetSubmission(string submissionId) {
        if (_token == null)
            throw new AuthenticationException(
                "You must authenticate before you can get a submission.");

        Submission = (Submission)SubmissionService.GetSubmission(submissionId);
        Submission.Metadata =
            JsonConvert.DeserializeObject<Metadata>(
                Encryption.DecryptData(Submission.EncryptedMetadata));
        Submission.Data =
            JsonConvert.DeserializeObject<Data>(Encryption.DecryptData(Submission.EncryptedData));

        return this;
    }

    /// <summary>
    /// Returns a list of all submissions for the given destination.
    /// </summary>
    /// <param name="destinationId"></param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSubscriber GetSubmissions(string destinationId) {
        if (_token == null)
            throw new AuthenticationException("You must call authenticate first.");

        var submissionsResult = SubmissionService.ListSubmissions(destinationId, 0, 100).Result;
        AvailableSubmissions = submissionsResult.TotalCount;

        // Creating a dictionary of destinationId to submissionIds from the REST API result
        Submissions = submissionsResult.Submissions
            .Select(s => new { s.DestinationId, s.SubmissionId })
            .ToDictionary(i => i.DestinationId,
                i =>
                    submissionsResult.Submissions
                        .Where(s => s.DestinationId == i.DestinationId)
                        .Select(d => d.SubmissionId));

        return this;
    }

    /// <summary>
    /// Reading attachments for a submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public FluentSubscriber GetAttachments() {
        // TODO add guard calls

        var attachments = new List<Attachment>();
        foreach (var id in Submission!.Attachments.Select(a => a.Id)) {
            var encryptedAttachment = SubmissionService.GetAttachment(Submission.Id, id);

            var content = Encryption.DecryptData(Convert.FromBase64String(encryptedAttachment));
            
            // TODO where do I get the hash from the server to verify the attachment?
            var hash = MD5.Create(HashAlgorithmName.SHA512.ToString())?.ComputeHash(content) ??
                       Array.Empty<byte>();
            
            attachments.Add(new Attachment(id, content, Convert.ToBase64String(hash), "dummy.pdf"));
        }

        Submission.Attachments = attachments;
        return this;
    }


    public FluentSubscriber ConfirmSubmission() {
        // TODO add guard calls
        return this;
        throw new NotImplementedException();
        SubmissionService.SubmitSubmission(Submission.Id, (SubmitSubmissionDto)Submission);

        return this;
    }
}
